/*
*************************************************************************************************
*****																							*
*****							T A S T A T U R A B F R A G E									*
*****																							*
*****							07.2005 - 07.2005  Thilo Mlls									*
*****																							*
*****	Abfrage einer 4x4 - Matrix - Tastatur													*
*****		 																					*
*****		 																					*
*****	_______________________________________________________________________________________	*
*****	Quellcode fr GNU-C-Compiler (AVR-GCC)													*
*************************************************************************************************
_________________________________________________________________________________________________
| Belegung der Ein-/ Ausgnge:																	|
| ----------------------------																	|
| PORT A: 	Bit 0: Pin 40: Gradientenmessung 0..+10V											|	
| -------	Bit 1: Pin 39: Gradientenmessung 0..-10V											|
|			Bit 2: Pin 38: Strommessung Relaiskontakt											|
|			Bit 3: Pin 37: Betriebsart-Umschalter '0'											|
|			Bit 4: Pin 36: Betriebsart-Umschalter '1'											|
|			Bit 5: Pin 35: LED 'Busy'															|
|			Bit 6: Pin 34: Schalter 'Mono-/Bistabil'											|
|			Bit 7: Pin 33: Relais K2 (ffner/Schlieer-Messung)									|
| PORT B: 	Bit 0: Pin 01: Relais K1 (Strommessung/Zeitmessung)									|
| -------	Bit 1: Pin 02: Umschalter 'Volt/min.' - 'Volt/sec.'									|
|			Bit 2: Pin 03: (INT2) Zeitmess-Eingang												|
|			Bit 3: Pin 04: (OC0) Ansteuerung ffner-Spule 'A2'									|
|			Bit 4: Pin 05: Taster 'Relais Prfung starten'										|
|			Bit 5: Pin 06: ISP - MOSI															|
|			Bit 6: Pin 07: ISP - MISO															|
|			Bit 7: Pin 08: ISP - SCK															|
| PORT C: 	Bit 0: Pin 22: Datenbus																|
| -------	Bit 1: Pin 23: Datenbus																|
|			Bit 2: Pin 24: Datenbus																|
|			Bit 3: Pin 25: Datenbus																|
|			Bit 4: Pin 26: Datenbus																|
|			Bit 5: Pin 27: Datenbus																|
|			Bit 6: Pin 28: Datenbus																|
|			Bit 7: Pin 29: Datenbus																|
| PORT D: 	Bit 0: Pin 14: RX RS232																|
| -------	Bit 1: Pin 15: TX RS232																|
|			Bit 2: Pin 16: Taster 'RESET' (INT0)												|
|			Bit 3: Pin 17: Taster 'Rel. prfen' (INT1)											|
|			Bit 4: Pin 18: Display 'E'															|
|			Bit 5: Pin 19: Display 'RW'															|
|			Bit 6: Pin 20: Display 'RS'															|
|			Bit 7: Pin 21: (OC2) Ansteuerung Schlieer-Spule 'A1'								|
|________________________________________________________________________________________________
*/
#include <avr/signal.h>
#include <avr/io.h>
#include <stdio.h>
#include <inttypes.h>
#include <stdlib.h>
#include <avr/eeprom.h>
#define F_CPU 16.000E6						// Prozessortakt fr delay festlegen
#include <avr/delay.h>
#include <scankey.c>

#define _EEPUT(addr, val) eeprom_write_byte ((uint8_t *)(addr), (uint8_t)(val)) 
#define _EEGET(var, addr) (var) = eeprom_read_byte ((uint8_t *)(addr)) 

// Bitdefinitionen des Status-Bytes


//  Globale Variablen im SRAM

volatile unsigned char Status = 0;
volatile unsigned char Temp = 0;
volatile unsigned char R_Byte = 0;
volatile unsigned char TX_Byte = ' ';

int USART_Transmit(char txtemp)				// RS232-Senderoutine
{

while ( !( UCSRA & (1<<UDRE))); 				// warten, bis Sendepuffer leer ist
UDR = txtemp;									// Byte senden

return 0;
}

int main (void)								// Start
{	

/***************************************************************************************/
/*** Start Tastaturabfrage *************************************************************/
/***************************************************************************************/

/***************************************************************************************/
// Initialisierung
/***************************************************************************************/
	
// Datenrichtung der Ports	
	
	DDRA = 0b11110000;					// Belegung siehe oben
	PORTA = 0b00001111;
	DDRB = 0b00000000;
	PORTB = 0b00000000;
	DDRC = 0b00000000;
	PORTC = 0b00000000;
	DDRD = 0b10000000;
	PORTD = 0b00000000;
	
// UART Einstellungen

	UCSRA = 0b00000000;
	UCSRB = 0b10011000;					// RX & TX enable, RX-Int. enabled
	UBRRL = 51;							// 19200 Baud bei 16 MHz, 0.2% Fehler
	

// TWI Einstellungen	
	
	TWBR = 16;
	TWSR = 2;
	TWCR = 0b00000000;					// TWI und TWI - Int. disabled
	
// Analog Comparator Einstellungen

	ACSR = 0b00000000;					// Analog Comparator enabled, rising edge
	
// Einstellungen fr AD-Wandler	
	
	ADCSRA = 0b00000000;				// 125kHz bei 16 MHz
	ADMUX = 0b00000000;					// ADC-Kanal 0, ADC enabled
	
// Einstellungen fr Timer 1

	TCCR1A = 0b00000000;
	TCCR1B = 0b00000000;				// Prescaler /1024, CTC enablel
	TIMSK = 0b00000000;					// Output-compare1-ovf enabled
	
	OCR1A = 15624;						// OCR1A = 15625 (16MHz / 1024 / 15625 = 1s)
	
// Einstellungen fr Interrupts und Sleepmode
	
	MCUSR = 0b00000000;
	MCUCR = 0b00000000;					// INT0 fallende Flanke
	GIMSK = 0b00000000;
	GIFR = 0b11100000;					// INT-Flags lschen
	GICR = 0b00000000;					// INT0 enabled
	
fclose (stdout);
fdevopen(USART_Transmit, NULL, 0);		// Output-Stream auf RS232-Ausgabe
		
for (;;)
	{
		TX_Byte = _scan_key (0);
		if ((TX_Byte != ' '))
		{
			PORTD |= (1<<PD7);
			
			if ((TX_Byte == 'E'))
				{
					printf ("\r\n");			// CR ausgeben
					continue;
				}
		
			printf ("%c",TX_Byte);			// Messwert ausgeben
			
			PORTD &= ~(1<<PD7);
		}
		
	}
	
/***************************************************************************************/	
/***************************************************************************************/
/*** Ende Tastaturabfrage **************************************************************/
/***************************************************************************************/
}

/***************************************************************************************/
/*** Ende main *************************************************************************/
/***************************************************************************************/

SIGNAL (SIG_UART_RECV)			// RX Interrupt-complete-Routine
{
	R_Byte = UDR;				// Empfangenes Byte in Globale Variable einlesen
}

/***************************************************************************************/
/***************************************************************************************/


